@@ -21,27 +21,13 @@
  *
  */
 
-#define MAX_PATH_LENGTH 64
-
 void dict_append_entry(DBusMessageIter *dict,
 			const char *key, int type, void *val);
-
 void dict_append_array(DBusMessageIter *dict, const char *key, int type,
 			void *val, int n_elements);
 
-dbus_bool_t emit_property_changed(DBusConnection *conn,
-					const char *path,
-					const char *interface,
-					const char *name,
-					int type, void *value);
-
-dbus_bool_t emit_array_property_changed(DBusConnection *conn,
-					const char *path,
-					const char *interface,
-					const char *name,
-					int type, void *value, int num);
-
 void set_dbus_connection(DBusConnection *conn);
-DBusConnection *get_dbus_connection(void);
+DBusConnection *btd_get_dbus_connection(void);
 
 const char *class_to_icon(uint32_t class);
+const char *gap_appearance_to_icon(uint16_t appearance);
