@@ -38,11 +38,12 @@ typedef enum {
 struct main_opts {
 	char		*name;
 	uint32_t	class;
+	gboolean	pairable;
 	uint32_t	pairto;
 	uint32_t	discovto;
 	uint8_t		privacy;
 
-	gboolean	reverse_sdp;
+	gboolean	reverse_discovery;
 	gboolean	name_resolv;
 	gboolean	debug_keys;
 	gboolean	fast_conn;
@@ -54,8 +55,9 @@ struct main_opts {
 
 	bt_mode_t	mode;
 	bt_gatt_cache_t gatt_cache;
+	uint16_t	gatt_mtu;
 
-	uint8_t		min_enc_key_size;
+	uint8_t		key_size;
 };
 
 extern struct main_opts main_opts;
