@@ -28,8 +28,10 @@
 #include <config.h>
 #endif
 
+#define _GNU_SOURCE
 #include <fcntl.h>
 #include <unistd.h>
+#include <stdlib.h>
 #include <sys/types.h>
 #include <string.h>
 
