@@ -25,6 +25,7 @@
 #include <config.h>
 #endif
 
+#define _GNU_SOURCE
 #include <stdio.h>
 #include <ctype.h>
 #include <stdarg.h>
@@ -525,6 +526,8 @@ static const struct {
 	{ 0x2adc, "Mesh Provisioning Data Out"			},
 	{ 0x2add, "Mesh Proxy Data In"				},
 	{ 0x2ade, "Mesh Proxy Data Out"				},
+	{ 0x2b29, "Client Supported Features"			},
+	{ 0x2b2A, "Database Hash"				},
 	/* vendor defined */
 	{ 0xfeff, "GN Netcom"					},
 	{ 0xfefe, "GN ReSound A/S"				},
@@ -1010,18 +1013,18 @@ int strsuffix(const char *str, const char *suffix)
 	int suffix_len;
 
 	if (!str || !suffix)
-		return false;
+		return -1;
 
 	if (str[0] == '\0' && suffix[0] != '\0')
-		return false;
+		return -1;
 
 	if (suffix[0] == '\0' && str[0] != '\0')
-		return false;
+		return -1;
 
 	len = strlen(str);
 	suffix_len = strlen(suffix);
 	if (len < suffix_len)
-		return false;
+		return -1;
 
 	return strncmp(str + len - suffix_len, suffix, suffix_len);
 }
