@@ -49,7 +49,7 @@ struct test_pdu {
 	bool fragmented;
 	bool continuing;
 	bool browse;
-	const uint8_t *data;
+	uint8_t *data;
 	size_t size;
 };
 
@@ -74,7 +74,7 @@ struct context {
 #define raw_pdu(args...)					\
 	{							\
 		.valid = true,					\
-		.data = data(args),				\
+		.data = g_memdup(data(args), sizeof(data(args))), \
 		.size = sizeof(data(args)),			\
 	}
 
@@ -82,7 +82,7 @@ struct context {
 	{							\
 		.valid = true,					\
 		.browse = true,					\
-		.data = data(args),				\
+		.data = g_memdup(data(args), sizeof(data(args))), \
 		.size = sizeof(data(args)),			\
 	}
 
@@ -90,7 +90,7 @@ struct context {
 	{							\
 		.valid = true,					\
 		.fragmented = true,				\
-		.data = data(args),				\
+		.data = g_memdup(data(args), sizeof(data(args))), \
 		.size = sizeof(data(args)),			\
 	}
 
@@ -98,7 +98,7 @@ struct context {
 	{							\
 		.valid = true,					\
 		.continuing = true,				\
-		.data = data(args),				\
+		.data = g_memdup(data(args), sizeof(data(args))), \
 		.size = sizeof(data(args)),			\
 	}
 
@@ -113,16 +113,14 @@ struct context {
 		tester_add(name, &data, NULL, function, NULL);		\
 	} while (0)
 
-static void test_debug(const char *str, void *user_data)
-{
-	const char *prefix = user_data;
-
-	tester_debug("%s%s", prefix, str);
-}
-
 static void test_free(gconstpointer user_data)
 {
 	const struct test_data *data = user_data;
+	struct test_pdu *pdu;
+	int i;
+
+	for (i = 0; (pdu = &data->pdu_list[i]) && pdu->valid; i++)
+		g_free(pdu->data);
 
 	g_free(data->test_name);
 	g_free(data->pdu_list);
@@ -164,12 +162,13 @@ static gboolean send_pdu(gpointer user_data)
 
 	pdu = &context->data->pdu_list[context->pdu_offset++];
 
-	if (pdu->browse)
+	if (pdu->browse) {
 		len = write(context->browse_fd, pdu->data, pdu->size);
-	else
+		tester_monitor('<', 0x0000, 0x001b, pdu->data, len);
+	} else {
 		len = write(context->fd, pdu->data, pdu->size);
-
-	util_hexdump('<', pdu->data, len, test_debug, "AVRCP: ");
+		tester_monitor('<', 0x0000, 0x0017, pdu->data, len);
+	}
 
 	g_assert_cmpint(len, ==, pdu->size);
 
@@ -215,8 +214,7 @@ static gboolean test_handler(GIOChannel *channel, GIOCondition cond,
 
 	g_assert(len > 0);
 
-	if (g_test_verbose())
-		util_hexdump('>', buf, len, test_debug, "AVRCP: ");
+	tester_monitor('>', 0x0000, 0x0017, buf, len);
 
 	if (!pdu->continuing)
 		g_assert_cmpint(len, ==, pdu->size);
@@ -254,7 +252,7 @@ static gboolean browse_test_handler(GIOChannel *channel, GIOCondition cond,
 
 	g_assert(len > 0);
 
-	util_hexdump('>', buf, len, test_debug, "AVRCP: ");
+	tester_monitor('>', 0x0000, 0x001b, buf, len);
 
 	g_assert_cmpint(len, ==, pdu->size);
 
