@@ -20,7 +20,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
 #include <sys/types.h>
@@ -188,6 +188,11 @@ send:
 		goto failed;
 	}
 
+	if (d->count >= TEST_BUF_MAX) {
+		g_print("io_cb count %u\n", d->count);
+		goto failed;
+	}
+
 	if (d->recv[d->count].len < 0 || (gssize) expect_len < 0)
 		return test_io_cb(io, G_IO_OUT, user_data);
 
